@@ -29,11 +29,14 @@ class Daikin:
         "ver",
         "type",
         "today_runtime",
+        "today_power_consumption",
         "current_month_power_consumption",
         "price_int",
         "compressor_frequency",
         "inside_temperature",
         "outside_temperature",
+        "wifi_settings",
+        "datetime",
     ]
 
     _host = None
@@ -46,25 +49,29 @@ class Daikin:
         self._host = host
 
     def _get(self, path):
-        """ Internal function to connect to and get any information"""
-        response = requests.get("http://" + self._host + path)
+        """Internal function to connect to and get any information"""
+        response = requests.get("http://" + self._host + path, timeout=10)
         response.raise_for_status()
         logging.debug(response.text)
-        if not len(response.text) > 0 or not response.text[0:4] == "ret=":
+        if (
+            not len(response.text) > 0
+            or not response.text[0:4] == "ret="
+            or response.text[0:6] == "ret=NG"
+        ):
             return None
         fields = {}
         for group in response.text.split(","):
             element = group.split("=")
-            if element[0] == "name":
+            if element[0] in ("name", "key"):
                 fields[element[0]] = urllib.parse.unquote(element[1])
             else:
                 fields[element[0]] = element[1]
         return fields
 
     def _set(self, path, data):
-        """ Internal function to connect to and update information"""
+        """Internal function to connect to and update information"""
         logging.debug(data)
-        response = requests.get("http://" + self._host + path, data)
+        response = requests.get("http://" + self._host + path, data, timeout=10)
         response.raise_for_status()
         logging.debug(response.text)
 
@@ -87,21 +94,32 @@ class Daikin:
         """
         return self._get("/common/get_notify")
 
-    def _get_week(self):
+    def _get_week(self, ex=False):
         """
-        Example:
+        Example (ex=False):
         ret=OK,today_runtime=601,datas=0/0/0/0/0/0/1000
+            (datas: values in Watts, last day last)
+        Example (ex=True):
+        ret=OK,s_dayw=2,week_heat=10/0/0/0/0/0/0/0/0/0/0/0/0/0,week_cool=0/0/0/0/0/0/0/0/0/0/0/0/0/0
+            (week_*: values in 100Watts, last day first)
         :return: dict
         """
-        return self._get("/aircon/get_week_power")
+        return self._get("/aircon/get_week_power" + ("_ex" if ex else ""))
 
-    def _get_year(self):
+    def _get_year(self, ex=False):
         """
-        Example:
-        ret=OK,previous_year=0/0/0/0/0/0/0/0/0/0/0/0,this_year=0/0/0/1
+        Example (ex=False):
+        ret=OK,previous_year=0/0/0/0/0/0/0/0/0/0/0/0,this_year=0/0/0/0/0/0/0/0/0/1
+            (*_year: values in 100Watts per month (jan-dec))
+        Example (ex=True):
+            ret=OK,curr_year_heat=0/0/0/0/0/0/0/0/0/0/0/1,
+            prev_year_heat=0/0/0/0/0/0/0/0/0/0/0/0,
+            curr_year_cool=0/0/0/0/0/0/0/0/0/0/0/0,
+            prev_year_cool=0/0/0/0/0/0/0/0/0/0/0/0
+            (*_year_*: values in 100Watts per month (jan-dec))
         :return: dict
         """
-        return self._get("/aircon/get_year_power")
+        return self._get("/aircon/get_year_power" + ("_ex" if ex else ""))
 
     def _get_target(self):
         """
@@ -164,6 +182,40 @@ class Daikin:
         """
         return self._get("/common/get_remote_method")
 
+    def _get_wifi(self):
+        """
+        Example:
+        ret=OK,ssid=wireless_ssid,security=mixed,key=%77%69%66%69%6b%65%79,link=1
+        :return: dict
+        """
+        return self._get("/common/get_wifi_setting")
+
+    def _get_datetime(self):
+        """
+        Example:
+        ret=OK,sta=1,cur=2022/12/01 22:01:02,reg=eu,dst=1,zone=10
+        :return:
+        """
+        return self._get("/common/get_datetime")
+
+    def _do_reboot(self):
+        return self._get("/common/reboot")
+
+    def _set_wifi(self, ssid, key, do_reboot=True):
+        """
+        Set the wifi settings
+        :param ssid: ssid of the new network
+        :param key: key of the new network
+        :param do_reboot: boolean indicating whether to reboot, to activate the settings
+        :return: None
+        """
+        key_encoded = "".join("%" + hex(ord(c))[2:].rjust(2, "0") for c in key)
+        data = {"ssid": ssid, "key": key_encoded, "security": "mixed"}
+        self._set("/common/set_wifi_setting", data)
+        if do_reboot:
+            res = self._do_reboot()
+            logging.debug("Reboot ordered to activate wifi changes: %s", res)
+
     @property
     def power(self):
         """
@@ -216,6 +268,25 @@ class Daikin:
         """
         return int(self._get_control()["f_dir"])
 
+    @property
+    def wifi_settings(self):
+        """
+        wifi settings
+        :return: tuple containing ssid and key of wifi network
+        """
+        wifi = self._get_wifi()
+        return wifi["ssid"], wifi["key"]
+
+    @property
+    def datetime(self):
+        """
+        datetime on the device
+        :return: string of datetime on the device (yyyy/mm/dd HH:MM:SS),
+            or None if not retrievable
+        """
+        datetime = self._get_datetime()["cur"]
+        return datetime if datetime != "-" else None
+
     @power.setter
     def power(self, value):
         self._control_set("pow", value)
@@ -240,6 +311,11 @@ class Daikin:
     def fan_direction(self, value):
         self._control_set("f_dir", value)
 
+    @wifi_settings.setter
+    def wifi_settings(self, value):
+        ssid, key = value
+        self._set_wifi(ssid, key)
+
     def _control_set(self, key, value):
         """
         set a get_control() item via one of the property.setters
@@ -302,13 +378,52 @@ class Daikin:
         """
         return int(self._get_week()["today_runtime"])
 
+    def _today_power_consumption_ex(self, ex=True, mode="heat"):
+        """
+        unit power consumption today (in Watts)
+        :param ex: boolean indicating whether to take form '_ex'
+        :param mode: string from ("heat", "cool") describing mode of operation;
+            ignored if ex==False
+        :return: Watts of power consumption
+        """
+        assert not ex or mode in (
+            "heat",
+            "cool",
+        ), 'mode should be from ("heat", "cool") if ex==True'
+        res = self._get_week(ex=ex)
+        if res is None:
+            return None
+        res = int(res[f"week_{mode}" if ex else "datas"].split("/")[0 if ex else -1])
+        return res * 100 if ex else res
+
+    @property
+    def today_power_consumption(self):
+        """
+        unit power consumption today (in Watts)
+        :return: Watts of power consumption
+        """
+        return self._today_power_consumption_ex(ex=False, mode=None)
+
+    def _month_power_consumption(self, month=None):
+        """
+        energy consumption
+        :param month: request a particular month-of-year (january=1);
+            None defaults to current month
+        :return: current-of-year energy consumption in kWh or None if not retrievable
+        """
+        if month is None:
+            if self.datetime is None:
+                return None
+            month = int(self.datetime.split("/")[1])
+        return int(self._get_year()["this_year"].split("/")[month - 1]) / 10.0
+
     @property
     def current_month_power_consumption(self):
         """
         energy consumption
-        :return: current month to date energy consumption in kWh
+        :return: current month to date energy consumption in kWh or None if not retrievable
         """
-        return int(self._get_year()["this_year"].split("/")[-1])
+        return self._month_power_consumption()
 
     @property
     def price_int(self):
@@ -322,7 +437,7 @@ class Daikin:
     def compressor_frequency(self):
         """
         compressor frequency/power
-        :return:
+        :return: The outside compressor load, normally between 12 to 84 max
         """
         return int(self._get_sensor()["cmpfreq"])
 
@@ -334,6 +449,14 @@ class Daikin:
         """
         return float(self._get_sensor()["htemp"])
 
+    @property
+    def inside_humidity(self):
+        """
+        inside relative humidity
+        :return: percent
+        """
+        return float(self._get_sensor()["hhum"])
+
     @property
     def outside_temperature(self):
         """
@@ -358,9 +481,9 @@ class Daikin:
         fields.update(self._get_control())
         fields.update(self._get_model())
         fields.update(self._get_remote())
+        fields.update(self._get_wifi())
+        fields.update(self._get_datetime())
         return fields
 
     def __str__(self):
-        return "Daikin(host={0},name={1},mac={2})".format(
-            self._host, self.name, self.mac
-        )
+        return f"Daikin(host={self._host},name={self.name},mac={self.mac})"
